@@ -1,10 +1,11 @@
 `impRZilr` <-
   function(x, maxit=10, eps=0.1, method="pls", 
-           dl=rep(0.05, ncol(x)), nComp = "boot", 
+           dl=rep(0.05, ncol(x)), 	nComp = "boot", 
            bruteforce=FALSE,  noisemethod="residuals", 
            noise=FALSE, R=10,
            verbose=FALSE){
-
+    
+    
     if( is.vector(x) ) stop("x must be a matrix or data frame")
     ## check if only numeric variables are in x:
     cl <- lapply(x, class)
@@ -12,27 +13,44 @@
     stopifnot((method %in% c("lm", "MM", "pls")))
     if( length(dl) < ncol(x)) stop(paste("dl has to be a vector of ", ncol(x)))
     if(method=="pls" & ncol(x)<5) stop("too less variables/parts for method pls")
-    if(!(nComp[1] %in% c("boot","cv"))){
-      if(length(nComp) != ncol(x)) stop("nComp must be numeric of length ncol(x) or boot or cv")
-    }
-
+    if(is.null(nComp)){
+	  pre <- FALSE
+	  nC <- NULL
+    } else if(nComp=="boot"){
+	  nC <- integer(ncol(x))
+	  pre <- TRUE
+	} else if(length(nComp) == ncol(x)){
+	  nC <- nComp
+	  pre <- FALSE
+	} else  {
+	  pre <- FALSE	
+	}
+#     pre <- TRUE
+#      if(length(nComp) != ncol(x) & nComp!="boot") stop("nComp must be NULL, boot or of length ncol(x)")
+#    } else if(nComp == "boot"){#
+#		pre <- TRUE
+#	} else {
+#		pre <- FALSE
+#	}
+    
     #################
     ## store rowSums
-    rs <- rowSums(x, na.rm=TRUE)
+    rs <- rowSums(x)
     
     #################
     ## zeros to NA:
     # check if values are in (0, dl[i]):
     check <- logical(ncol(x))
     for(i in 1:ncol(x)){
-      check[i] <- any(x[,i] < dl[i] & x[,i] != 0)
+#      check[i] <- any(x[,i] < dl[i] & x[,i] != 0)
       x[x[,i] < dl[i],i] <- 0
     }
-    if(any(check)){warning("values below detection limit have been set to zero and will be imputed")}
+#    if(any(check)){warning("values below detection limit have been set to zero and will be imputed")}
     check2 <- any(x < 0)
     if(check2){warning("values below 0 set have been set to zero and will be imputed")}
     x[x == 0] <- NA
     x[x < 0] <- NA
+    indexFinalCheck <- is.na(x)
     
     ################
     ## sort variables of x based on 
@@ -42,72 +60,56 @@
     o <- order(wcol)
     x <- x[,o]
     if(verbose) cat("variables with decreasing number of missings:", o)
-    
+    ## --> now work in revised order of variables
+    ## dl must also be in correct order
+    dlordered <- dl[o]
     
     #################
     ## index of missings / non-missings
     w <- is.na(x)
     wn <- !is.na(x)
-	## lines with RZ:
-    # w2 <- apply(x, 1, function(x){ sum(is.na(x)) })
-    # indNA <- apply(x, 2, function(x){any(is.na(x))})
+#    w2 <- apply(x, 1, function(x){ sum(is.na(x)) })
+    #	indNA <- apply(x, 2, function(x){any(is.na(x))})
     
     #################
+    ## sort the columns of the data according to the amount of missings in the variables
 #    wcol <- apply(x, 2, function(x) length(which(is.na(x))))
 #    indM <- sort(wcol, index.return=TRUE, decreasing=TRUE)$ix
-	# number of RZ in each sorted variable
-     nwcol <- apply(x, 2, function(x) length(which(is.na(x))))
-#    indM <- sort(wcol, index.return=TRUE, decreasing=TRUE)$ix
-	
-	## save orig data for later use:    
-	xOrig <- x
-     
+    xcheck <- x
+    w2 <- is.na(x)
+    
+    
     ################
     ## initialisation
-	# cols with RZ's:
     indNA <- apply(x, 2, function(x){any(is.na(x))})
-	# initialize RZ's with 2/3 dl per column
-    for(i in which(indNA)){
-      x[w[,i],i] <- dl[i]*2/3 #*runif(sum(ind),1/3,2/3)
+    for(i in 1:length(dl)){
+      ind <- is.na(x[,i])
+      #		if(length(ind) > 0) x[ind,i] <- dl[i]*runif(sum(ind),1/3,2/3)
+      if(length(ind) > 0) x[ind,i] <- dlordered[i] *2/3
     }
-
+    xOrig <- x
     
     ################
     n <- nrow(x) 
     d <- ncol(x)
-    
-    ### create progress bar
-    pb <- txtProgressBar(min = 0, max = maxit, style = 3,
-                         title=paste("maximal time for ", maxit, "iterations"))
-    ii <- 1
-    
     ###  start the iteration
     if(verbose) cat("\n start the iteration:")
     it <- 1; criteria <- 99999999
-    if(length(nComp) > 1) nC <- nComp else nC <- integer(length(which(indNA)))
-    
     
     while(it <= maxit & criteria >= eps){
       if(verbose) cat("\n iteration", it, "; criteria =", criteria)	
-	  ## for criteria used:
       xold <- x  
-	  ## inner loop:
       for(i in which(indNA)){
         if(verbose) cat("\n replacement on part", i)
-    		## ensure that imputed values are not too close to zero:
-    		x[x < 2*.Machine$double.eps] <- 2*.Machine$double.eps
-        ## transformation of the detection limit:
-        phi <- -isomLR(cbind(rep(dl[i], n), x[,-i,drop=FALSE]))[,1] 
-        ## transformation of the data, variable i on first column:
+        ## detection limit in ilr-space
+        phi <- -isomLR(cbind(rep(dlordered[i], n), x[,-i,drop=FALSE]))[,1] 
+        #		part <- cbind(x[,i,drop=FALSE], x[,-i,drop=FALSE])
+        x[x < 2*.Machine$double.eps] <- 2*.Machine$double.eps
         xilr <- data.frame(-isomLR(cbind(x[,i,drop=FALSE], x[,-i,drop=FALSE])))
-    		## ensure that first variable is fixed:
-        # c1 <- colnames(xilr)[1]					
-        # colnames(xilr)[1] <- "V1"	
-    		## response:
+        c1 <- colnames(xilr)[1]					
+        colnames(xilr)[1] <- "V1"	
         response <- as.matrix(xilr[,1,drop=FALSE])
-    		## predictors (everything except response):
         predictors <- as.matrix(xilr[,-1,drop=FALSE])
-    		## fit and prediction:
         if(method=="lm"){ 
           reg1 <- lm(response ~ predictors)
           yhat <- predict(reg1, new.data=data.frame(predictors))
@@ -115,57 +117,42 @@
           reg1 <- rlm(response ~ predictors, method="MM",maxit = 100)#rlm(V1 ~ ., data=xilr2, method="MM",maxit = 100)
           yhat <- predict(reg1, new.data=data.frame(predictors))
         } else if(method=="pls"){
-          if(it == 1 & nComp[1] =="boot"){ 
-        			## evaluate ncomp in the first run:
-        			nC[i] <- bootnComp(predictors,response, R, plotting=TRUE)$res
- #            nC[i] <- bootnComp(xilr[,!(colnames(xilr) == "V1"),drop=FALSE],y=xilr[,"V1"], R, plotting=TRUE)$res2
+          if(it == 1 & pre){ ## evaluate ncomp.
+            nC[i] <- bootnComp(xilr[,!(colnames(xilr) == "V1"),drop=FALSE],y=xilr[,"V1"], R, 
+					      plotting=FALSE)$res #$res2
           }
-          if(it == 1 & nComp[1] == "cv"){ 
-            ## evaluate ncomp in the first run:
-            stop("implement method cv")
-            nC[i] <- mvr(as.matrix(response) ~ as.matrix(predictors), 
-                            method="simpls")
-          }          
-          if(verbose) cat("   ;   ncomp:",nC[i])
-          reg1 <- mvr(as.matrix(response) ~ as.matrix(predictors), 
-                      ncomp=nC[i], method="simpls")
-          yhat <- predict(reg1, new.data=data.frame(predictors), 
-                          ncomp=nC[i])
+          if(verbose) cat("   ;   ncomp:",nComp[i])
+          reg1 <- mvr(as.matrix(response) ~ as.matrix(predictors), ncomp=nC[i], method="simpls")
+          yhat <- predict(reg1, new.data=data.frame(predictors), ncomp=nC[i])
         }
         
         #		s <- sqrt(sum(reg1$res^2)/abs(nrow(xilr)-ncol(xilr))) ## quick and dirty: abs()
         s <- sqrt(sum(reg1$res^2)/nrow(xilr)) 
         ex <- (phi - yhat)/s 
-        if(any(is.na(yhat)) | any(yhat == "NaN")) stop("here NA - yhat")
-        if(any(is.na(phi)) | any(phi == "NaN")) stop("here NA - phi")
-        if(any(is.na(ex)) | any(ex == "NaN")) stop("here NA - ex")
-        
         yhat2sel <- ifelse(dnorm(ex[w[, i]]) > .Machine$double.eps,
                            yhat[w[, i]] - s*dnorm(ex[w[, i]])/pnorm(ex[w[, i]]),
                            yhat[w[, i]])
         if(any(is.na(yhat)) || any(yhat=="Inf")) stop("Problems in ilr because of infinite or NA estimates")
         # check if we are under the DL:
-#        yhat2seltest <<- yhat2sel
-#        phitest <<- phi[w[,i]]
         if(any(yhat2sel >= phi[w[, i]])){
           yhat2sel <- ifelse(yhat2sel > phi[w[, i]], phi[w[, i]], yhat2sel)
         }
         xilr[w[, i], 1] <- yhat2sel
         xinv <- isomLRinv(-xilr)
-        ## reordering to previous order 
+        ## reordering of xOrig
         if(i %in% 2:(d-1)){
           xinv <- cbind(xinv[,2:i], xinv[,c(1,(i+1):d)])
         }
         if(i == d){
           xinv <- cbind(xinv[,2:d], xinv[,1])
         }
-        x <- adjust2(xinv, xOrig, w)
-
+ #       browser()
+        x <- adjustImputed(xinv, xOrig, w2)
         #		x <- adjust3(xinv, xOrig, w2) 
         #		## quick and dirty:
         #		x[!w] <- xOrig[!w]
       }
-	  setTxtProgressBar(pb, ii); ii <- ii + 1      
+      
       it <- it + 1
       criteria <- sum( ((xold - x)/x)^2, na.rm=TRUE) ## DIRTY: (na.rm=TRUE)
       if(verbose & criteria != 0) cat("\n iteration", it, "; criteria =", criteria)
@@ -180,12 +167,12 @@
         inderr <- w[,i]
         if(noisemethod == "residuals") {
           error <- sample(residuals( reg1 )[inderr], 
-                          size=nwcol[i], replace=TRUE)
+                          size=wcol[i], replace=TRUE)
           reg1$res[inderr] <- error
         } else {
           mu <- median(residuals( reg1 )[inderr])
           sigma <- mad(residuals( reg1 )[inderr])
-          error <- rnorm(nwcol[i], mean=mu, sd=sigma)
+          error <- rnorm(wcol[i], mean=mu, sd=sigma)
           reg1$res[inderr] <- error		   
         }
         # return realizations
@@ -218,79 +205,62 @@
       }
     }
     ### end add random error ###
-#    x <- adjust3(x, xOrig, w)
-    if(!all.equal(x[!w], xOrig[!w])) stop("adjust problems - revise algorithm") 
+ #   x <- adjust3(x, xOrig, w)
+ #   x[!w] <- xOrig[!w] 
     x <- x[,order(o)] ## checked: reordering is OK!
     colnames(x) <- cn
+    ## check if all is fine:
+    # check if values are in (0, dl[i]):
+    checkDL <- function(x, dl, indexNA){
+      check <- logical(ncol(x))
+      for(i in 1:ncol(x)){
+         check[i] <- any(x[indexNA[,i],i] > dl[i])
+         if(check[i]){ 
+           x[which(x[indexNA[,i],i] > dl[i]),i] <- dl[i]
+         }
+      }
+      if(any(check)){
+        message("few imputed values have been corrected")      
+      }
+      return(x)
+    }
+    x <- checkDL(x, dl, indexFinalCheck)
     
-    #   ## recover abs values with rs
-    #   xtest <<- x
-    #   w <<- w
-    #   x_0 <- x
-    #   x_0[w] <- 0
-    #   rs_imp <- rowSums(x_0)
-    #   fac <- rs/rs_imp
-    #   for(i in 1:ncol(x)){
-    #	   x[,i] <- x[,i] * fac[i]
-    #   }
-    
-    #  # quick and dirty: (preserve absolute values)
-    #  x[!w] <- xorig[!w] 
+  
     
     res <- list(x=x, criteria=criteria, iter=it, 
-                maxit=maxit, wind=w, nComp=nC, method=method)
+                maxit=maxit, wind=w, nComp=nC, method=method, dl=dl)
     class(res) <- "replaced"
     invisible(res)
   }
 
-cvnComp <- function(X,y, R=99, plotting=FALSE){
-  rr <- mvr(as.matrix(y) ~ as.matrix(X), 
-      method="simpls", validation="CV")$validation$PRESS
-  return(which.max(as.numeric(rr)))
-}
-  
 bootnComp <- function(X,y, R=99, plotting=FALSE){
   ind <- 1:nrow(X)
   d <- matrix(, ncol=R, nrow=nrow(X))#nrow(X))
   for(i in 1:R){
-    bootind <- sample(ind, replace=TRUE)
-	## "paired" bootstrap sample:
+    bootind <- sample(ind)
+    XX <- X
+    yy <- y
     ds <- cbind(X[bootind,], as.numeric(y[bootind]))
     colnames(ds)[ncol(ds)] <- "V1"
-    ## regression on bootstrap sample, in validation 
-	## the predictions and so called PRESS values are then stored:
-    reg1 <- mvr(V1~., data=data.frame(ds), method="simpls", validation="none")#, validation="CV")
-	## prediction error criteria is wrong
-	## (even the results looks fine), since we
-	## compare original values of response (y) from predictions
-	## based on bootstrap samples:
-#    d[1:reg1$ncomp,i] <- as.numeric(apply(reg1$validation$pred, 3, function(x) sum(((y - x)^2)) ) )
-		
-	## better? To use the coefficients from reg1 and predict 
-	## based on original data --> prediction error:
-	ppp <- predict(reg1, X)
-	d[1:reg1$ncomp,i] <- as.numeric(apply(ppp, 3, function(x) mean(abs(y - x)) ))	
+    reg1 <- mvr(V1~., data=data.frame(ds), method="simpls", validation="CV")
+    d[1:reg1$ncomp,i] <- as.numeric(apply(reg1$validation$pred, 3, function(x) sum(((y - x)^2)) ) )
   }
   d <- na.omit(d)
-  sdev <- apply(d, 1, quantile, probs=0.5, na.rm=TRUE)
-  sdev2 <- apply(d, 1, quantile, probs=0.25, na.rm=TRUE)
-  sdevs <- sdev -sdev2
-  means <- apply(d, 1, median, na.rm=TRUE)
-  sdev3 <- apply(d, 1, mad)
+  sdev <- apply(d, 1, sd, na.rm=TRUE)
+  means <- apply(d, 1, mean, na.rm=TRUE)
   mi <- which.min(means)
-  threshold <- means + 2*sdev3
-  res <- which.min(!(means < threshold[mi]) )
-  minsd <- means - sdevs > means[mi]
+  r <- round(ncol(X)/20)
+  mi2 <- which.min(means[r:length(means)])+r-1
+  minsd <- means - sdev > means[mi]
   check <- means
-  check[!minsd] <- 999999999999999
+  check[!minsd] <- 99999999
   if(plotting) plot(means, type="l")
-  res2 <- which.min(check)
-  list(res=res, res2=NULL)
+  res <- which.min(check)
+  list(res=res, res2=mi2)
 }
 
 
-
-
 bootnCompHD <- function(X,y, R=99, plotting=FALSE){
   ind <- 1:nrow(X)
   d <- matrix(, ncol=R, nrow=nrow(X))#nrow(X))
@@ -338,14 +308,29 @@ bootnCompHD <- function(X,y, R=99, plotting=FALSE){
 ## test adjust2:
 
 adjust2 <- function (xImp, xOrig, wind){
+  ## aim: do not change original values
+  ## adapt imputations
   xneu = xImp
   s1 <- rowSums(xOrig, na.rm = TRUE)
+  ## per row: consider rowsums of imputed data
+  ## example: 
+  ## wind: F F T F F
+  ## ganz orig:  3 5 NA 8 10   (sum=26)
+  ## orig(init): 3 5 6.5 8 10  (sum=32.5)
+  ## imp:        3 5 7 8 10    (sum=33)
+  ## s: 26
+  ## s2: 7
+  ## fac: 26/(26+7)
+  ## s1: 32.5/(26/(26+7)) =  41.25
   for (i in 1:nrow(xImp)) {
     if(any(wind[i,])) s <- sum(xImp[i, !wind[i, ]]) else s <- 1
     if(any(wind[i,])) s2 <- sum(xImp[i, wind[i, ]]) else s2 <- 0
+    # how much is rowsum increased by imputation:
     fac <- s/(s + s2)
+    # decrese rowsums of orig.
     s1[i] <- s1[i]/fac
   }
+  ## impS: 41.25/33
   impS <- s1/rowSums(xImp)
   for (i in 1:ncol(xImp)) {
     xneu[, i] <- xImp[, i] * impS
@@ -354,44 +339,12 @@ adjust2 <- function (xImp, xOrig, wind){
   return(xImp)
 }
 
-#x <- matrix(1:100, ncol=5)
-#x <- x[,c(2,1,3:5)]
-#colnames(x) <- c("eins","zwei","drei","vier","fuenf")
-#x[x< 10] <- 0
-#x[11,3] <- 0
-#x[20,3] <- 0
-#dl <- rep(10,5)
-#
-#set.seed(123)
-#x <- xorig <- constSum(genVarsSmall(mvrnorm(20, mu=rep(1,3), Sigma=diag(3)), 15)[,1:5],100)
-#x <- xorig <- x[order(x[,1]), ]
-#dl <- apply(x, 2, quantile, 0.1)
-#dl[1] <- 7.32
-#for(i in 1:5){
-#	x[x[,i] < dl[i], i] <- 0
-#}
-#colnames(x) <- c("eins","zwei","drei","vier","fuenf")
-#imp <- impRZilr(x, dl=dl, method="pls", eps=0.00001, maxit=100)
-#imp$x
-## ausfuerhen des codes der fkt
-#y <- x
-#x <- xOrig
-#adjust2(xinv, xOrig, w) ## OK
-#
-#
-#
-#y <- x+rnorm(100, 0, 0.05)
-#y[1,3] <- 8
-#a <- adjust2(y, x, wind)
-#x[1,1]/x[2,2]
-#a[1,1]/a[2,2]
-#res <- impRZilr(x)
 
 adjust3 <- function(xImp, xOrig, wind){
   xOrigSum <- rowSums(xOrig)
   # sum imputed without former zeros:
   xImpSum <- numeric(ncol(xOrig))
-  for(i in 1:ncol(xOrig)){
+  for(i in 1:nrow(xOrig)){
     xImpSum[i] <- sum(xImp[i,!wind[i,]])
     fac <- xOrigSum[i] / xImpSum[i]
     xImp[i,wind[i,]] <- xImp[i,wind[i,]]  * fac
@@ -977,264 +930,3 @@ adjust3 <- function(xImp, xOrig, wind){
 # 	invisible(res)
 # }
 # 
-
-# 
-#   `impRZilr2` <-
-#   function(x, maxit=10, eps=0.1, method="pls", 
-#            dl=rep(0.05, ncol(x)), 	nComp = NULL, 
-#            bruteforce=FALSE,  noisemethod="residuals", noise=TRUE, R=10,
-#            verbose=FALSE){
-#     
-#     
-#     if( is.vector(x) ) stop("x must be a matrix or data frame")
-#     stopifnot((method %in% c("lm", "MM", "pls")))
-#     if( length(dl) < ncol(x)) stop(paste("dl has to be a vector of ", ncol(x)))
-#     if(method=="pls" & ncol(x)<5) stop("too less variables/parts for method pls")
-#     if(!is.null(nComp)){
-#       pre <- TRUE
-#       if(length(nComp) != ncol(x)) stop("nComp mmmmmust be NULL or of length ncol(x)")
-#     } else pre <- FALSE
-#     
-#     #################
-#     ## zeros to NA:
-#     x[x==0] <- NA
-#     
-#     ################
-#     ## sort variables of x based on 
-#     ## decreasing number of missings in the variables
-#     wcol <- - abs(apply(x, 2, function(x) sum(is.na(x))))
-#     o <- order(wcol)
-#     x <- x[,o]
-#     if(verbose) cat("variables with decreasing number of missings:", o)
-#     
-#     
-#     #################
-#     ## index of missings / non-missings
-#     w <- is.na(x)
-#     wn <- !is.na(x)
-#     w2 <- apply(x, 1, function(x){ sum(is.na(x)) })
-#     #	indNA <- apply(x, 2, function(x){any(is.na(x))})
-#     
-#     #################
-#     ## sort the columns of the data according to the amount of missings in the variables
-#     wcol <- apply(x, 2, function(x) length(which(is.na(x))))
-#     indM <- sort(wcol, index.return=TRUE, decreasing=TRUE)$ix
-#     cn <- colnames(x)
-#     xcheck <- x
-#     
-#     
-#     ################
-#     ## initialisation
-#     indNA <- apply(x, 2, function(x){any(is.na(x))})
-#     for(i in 1:length(dl)){
-#       ind <- is.na(x[,i])
-#       if(length(ind) > 0) x[ind,i] <- dl[i]*runif(sum(ind),1/3,2/3)
-#     }
-#     
-#     xOrig <- x
-#     
-#     ################
-#     ## detection limit in ilr-space
-#     n <- nrow(x) 
-#     d <- ncol(x)
-#     ###  start the iteration
-#     if(verbose) cat("\n start the iteration:")
-#     it <- 1; criteria <- 99999999
-#     
-#     while(it <= maxit & criteria >= eps){
-#       if(verbose) cat("\n iteration", it, "; criteria =", criteria)	
-#       xold <- x  
-#       for(i in which(indNA)){
-#         if(verbose) cat("\n replacement on part", i)
-#         test <<- cbind(rep(dl[i], n), x[,-i,drop=FALSE])
-#         phi <- -isomLR(cbind(rep(dl[i], n), x[,-i,drop=FALSE]))[,1] 
-#         #		part <- cbind(x[,i,drop=FALSE], x[,-i,drop=FALSE])
-#         x[x < 2*.Machine$double.eps] <- 2*.Machine$double.eps
-#         xilr <- data.frame(-isomLR(cbind(x[,i,drop=FALSE], x[,-i,drop=FALSE])))
-#         c1 <- colnames(xilr)[1]					
-#         colnames(xilr)[1] <- "V1"	
-#         xilr <<- xilr
-#         response <- as.matrix(xilr[,1,drop=FALSE])
-#         predictors <- as.matrix(xilr[,-1,drop=FALSE])
-#         if(method=="lm"){ 
-#           reg1 <- lm(response ~ predictors)
-#           yhat <- predict(reg1, new.data=data.fram(predictors))
-#         } else if(method=="MM"){
-#           reg1 <- rlm(response ~ predictors, method="MM",maxit = 100)#rlm(V1 ~ ., data=xilr2, method="MM",maxit = 100)
-#           yhat <- predict(reg1, new.data=data.fram(predictors))
-#         } else if(method=="pls"){
-#           if(it == 1 & !pre){ ## evaluate ncomp.
-#             nComp[i] <- bootnComp(xilr[,!(colnames(xilr) == "V1"),drop=FALSE],y=xilr[,"V1"], R, plotting=TRUE)$res2
-#           }
-#           if(verbose) cat("   ;   ncomp:",nComp[i])
-#           reg1 <- mvr(as.matrix(response) ~ as.matrix(predictors), ncomp=nComp[i], method="simpls")
-#           yhat <- predict(reg1, new.data=data.fram(predictors), ncomp=nComp[i])
-#         }
-#         
-#         #		s <- sqrt(sum(reg1$res^2)/abs(nrow(xilr)-ncol(xilr))) ## quick and dirty: abs()
-#         s <- sqrt(sum(reg1$res^2)/nrow(xilr)) ## quick and dirty: abs()
-#         ex <- (phi - yhat)/s 
-#         yhat2sel <- ifelse(dnorm(ex[w[, i]]) > .Machine$double.eps,
-#                            yhat[w[, i]] - s*dnorm(ex[w[, i]])/pnorm(ex[w[, i]]),
-#                            yhat[w[, i]])
-#         if(any(is.na(yhat)) || any(yhat=="Inf")) stop("Problems in ilr because of infinite or NA estimates")
-#         # check if we are under the DL:
-#         if(any(yhat2sel >= phi[w[, i]])){
-#           yhat2sel <- ifelse(yhat2sel > phi[w[, i]], phi[w[, i]], yhat2sel)
-#         }
-#         xilr[w[, i], 1] <- yhat2sel
-#         xinv <- isomLRinv(-xilr)
-#         ## reordering of xOrig
-#         if(i %in% 2:(d-1)){
-#           xinv <- cbind(xinv[,2:i], xinv[,c(1,(i+1):d)])
-#         }
-#         if(i == d){
-#           xinv <- cbind(xinv[,2:d], xinv[,1])
-#         }
-#         
-#         x <- adjust2(xinv, xOrig, w) 
-#       }
-#       
-#       it <- it + 1
-#       criteria <- sum( ((xold - x)/x)^2, na.rm=TRUE) ## DIRTY: (na.rm=TRUE)
-#       if(verbose & criteria != 0) cat("\n iteration", it, "; criteria =", criteria)
-#     }
-#     
-#     #### add random error ###
-#     if(noise){
-#       for(i in which(indNA)){
-#         if(verbose) cat("\n add noise on variable", i)
-#         
-#         # add error terms
-#         inderr <<- w[,i]
-#         if(noisemethod == "residuals") {
-#           error <- sample(residuals( reg1 )[inderr], 
-#                           size=wcol[i], replace=TRUE)
-#           reg1$res[inderr] <- error
-#         } else {
-#           mu <- median(residuals( reg1 )[inderr])
-#           sigma <- mad(residuals( reg1 )[inderr])
-#           error <- rnorm(wcol[i], mean=mu, sd=sigma)
-#           reg1$res[inderr] <- error		   
-#         }
-#         # return realizations
-#         yhat[inderr] <- yhat[inderr] + error
-#         
-#         
-#         s <- sqrt(sum(reg1$res^2)/nrow(xilr)) ## quick and dirty: abs()
-#         ex <- (phi - yhat)/s 
-#         yhat2sel <- ifelse(dnorm(ex[w[, i]]) > .Machine$double.eps,
-#                            yhat[w[, i]] - s*dnorm(ex[w[, i]])/pnorm(ex[w[, i]]),
-#                            yhat[w[, i]])
-#         if(any(is.na(yhat)) || any(yhat=="Inf")) stop("Problems in ilr because of infinite or NA estimates")
-#         # check if we are under the DL:
-#         if(any(yhat2sel >= phi[w[, i]])){
-#           yhat2sel <- ifelse(yhat2sel > phi[w[, i]], phi[w[, i]], yhat2sel)
-#         }
-#         xilr[w[, i], 1] <- yhat2sel
-#         xinv <- isomLRinv(-xilr)
-#         ## reordering of xOrig
-#         if(i %in% 2:(d-1)){
-#           xinv <- cbind(xinv[,2:i], xinv[,c(1,(i+1):d)])
-#         }
-#         if(i == d){
-#           xinv <- cbind(xinv[,2:d], xinv[,1])
-#         }
-#         
-#         x <- adjust2(xinv, xOrig, w) 
-#       }
-#     }
-#     ### end add random error ###
-#     
-#     x <- x[,order(o)] ## checked: reordering is OK!
-#     colnames(x) <- colnames(xcheck)
-#     res <- list(x=x, criteria=criteria, iter=it, 
-#                 maxit=maxit, wind=w, nComp=nComp, method=method)
-#     class(res) <- "replaced"
-#     invisible(res)
-#   }
-# 
-# bootnComp <- function(X,y, R=99, plotting=FLASE){
-#   ind <- 1:nrow(X)
-#   d <- matrix(, ncol=R, nrow=nrow(X))#nrow(X))
-#   for(i in 1:R){
-#     bootind <- sample(ind)
-#     XX <- X
-#     yy <- y
-#     ds <- cbind(X[bootind,], as.numeric(y[bootind]))
-#     colnames(ds)[ncol(ds)] <- "V1"
-#     reg1 <- mvr(V1~., data=data.frame(ds), method="simpls", validation="CV")
-#     d[1:reg1$ncomp,i] <- as.numeric(apply(reg1$validation$pred, 3, function(x) sum(((y - x)^2)) ) )
-#   }
-#   d <- na.omit(d)
-#   sdev <- apply(d, 1, sd, na.rm=TRUE)
-#   means <- apply(d, 1, mean, na.rm=TRUE)
-#   mi <- which.min(means)
-#   r <- round(ncol(X)/20)
-#   mi2 <- which.min(means[r:length(means)])+r-1
-#   minsd <- means - sdev > means[mi]
-#   check <- means
-#   check[!minsd] <- 99999999
-#   if(plotting) plot(means, type="l")
-#   res <- which.min(check)
-#   list(res=res, res2=mi2)
-# }
-# 
-# 
-# bootnCompHD <- function(X,y, R=99, plotting=FALSE){
-#   ind <- 1:nrow(X)
-#   d <- matrix(, ncol=R, nrow=nrow(X))#nrow(X))
-#   for(i in 1:R){
-#     bootind <- sample(ind)
-#     XX <- X
-#     yy <- y
-#     ds <- cbind(X[bootind,], as.numeric(y[bootind]))
-#     colnames(ds)[ncol(ds)] <- "V1"
-#     reg1 <- mvr(V1~., data=data.frame(ds), method="simpls", validation="CV")
-#     d[1:reg1$ncomp,i] <- as.numeric(apply(reg1$validation$pred, 3, function(x) sum(((y - x)^2)) ) )
-#   }
-#   d <- na.omit(d)
-#   sdev <- apply(d, 1, mad, na.rm=TRUE)
-#   means <- apply(d, 1, median, na.rm=TRUE)
-#   mi <- which.min(means)
-#   if(plotting) plot(means, type="l", col="blue", ylab="squared total prediction error", xlab="number of components")
-#   themean <- mean(means)
-#   thesd <- sd(means)
-#   abovethreshold <- themean - sdev > means
-#   check <- means
-#   check[!abovethreshold] <- 9999999999
-#   res <- which.min(check)
-#   #	minsd <- means - sdev > means[mi]
-#   #	check <- means
-#   #	check[!minsd] <- 99999999
-#   #	res <- which.min(check)
-#   if(plotting){
-#     abline(v=res, lwd=3)
-#     abline(h=mi, col="red")
-#     #		abline(h=means-sdev, lty=3)
-#   }
-#   list(res=res, mean=means)
-# }
-# 
-# 
-# 
-# ## test adjust2:
-# 
-# adjust2 <- function (xImp, xOrig, wind){
-#   xneu = xImp
-#   s1 <- rowSums(xOrig, na.rm = TRUE)
-#   for (i in 1:nrow(xImp)) {
-#     if(any(wind[i,])) s <- sum(xImp[i, !wind[i, ]]) else s <- 1
-#     if(any(wind[i,])) s2 <- sum(xImp[i, wind[i, ]]) else s2 <- 0
-#     fac <- s/(s + s2)
-#     s1[i] <- s1[i]/fac
-#   }
-#   impS <- s1/rowSums(xImp)
-#   for (i in 1:ncol(xImp)) {
-#     xneu[, i] <- xImp[, i] * impS
-#   }
-#   xImp <- xneu
-#   return(xImp)
-# }
-
-
